@@ -4,7 +4,6 @@
 #include "graphics-hook.h"
 #include "../graphics-hook-ver.h"
 #include "../obfuscate.h"
-#include "../funchook.h"
 
 #define DEBUG_OUTPUT
 
@@ -68,12 +67,12 @@ bool init_pipe(void)
 	char new_name[64];
 	sprintf(new_name, "%s%lu", PIPE_NAME, GetCurrentProcessId());
 
-	if (!ipc_pipe_client_open(&pipe, new_name)) {
+	const bool success = ipc_pipe_client_open(&pipe, new_name);
+	if (!success) {
 		DbgOut("[OBS] Failed to open pipe\n");
-		return false;
 	}
 
-	return true;
+	return success;
 }
 
 static HANDLE init_event(const wchar_t *name, DWORD pid)
@@ -317,6 +316,7 @@ static inline bool attempt_hook(void)
 	//static bool ddraw_hooked = false;
 	static bool d3d8_hooked = false;
 	static bool d3d9_hooked = false;
+	static bool d3d12_hooked = false;
 	static bool dxgi_hooked = false;
 	static bool gl_hooked = false;
 #if COMPILE_VULKAN_HOOK
@@ -329,6 +329,12 @@ static inline bool attempt_hook(void)
 	}
 #endif //COMPILE_VULKAN_HOOK
 
+#if COMPILE_D3D12_HOOK
+	if (!d3d12_hooked) {
+		d3d12_hooked = hook_d3d12();
+	}
+#endif
+
 	if (!d3d9_hooked) {
 		if (!d3d9_hookable()) {
 			DbgOut("[OBS] no D3D9 hook address found!\n");
@@ -384,6 +390,24 @@ static inline bool attempt_hook(void)
 		}
 	}*/
 
+#if HOOK_VERBOSE_LOGGING
+	DbgOut("[OBS] Attempt hook: D3D8=");
+	DbgOut(d3d8_hooked ? "1" : "0");
+	DbgOut(", D3D9=");
+	DbgOut(d3d9_hooked ? "1" : "0");
+	DbgOut(", D3D12=");
+	DbgOut(d3d12_hooked ? "1" : "0");
+	DbgOut(", DXGI=");
+	DbgOut(dxgi_hooked ? "1" : "0");
+	DbgOut(", GL=");
+	DbgOut(gl_hooked ? "1" : "0");
+#if COMPILE_VULKAN_HOOK
+	DbgOut(", VK=");
+	DbgOut(vulkan_hooked ? "1" : "0");
+#endif
+	DbgOut("\n");
+#endif
+
 	return false;
 }
 
